@@ -53,15 +53,6 @@
 /************************************************************************
  *			  DEFINES AND MACROS				*
  ***********************************************************************/
-#if 0
-#if defined(__NetBSD__) || defined (__FreeBSD__) || defined (__OpenBSD__)
-# include <sdla_te1_pmc.h>
-#elif defined (__WINDOWS__)
-# include "sdla_te1_pmc.h"
-#else
-# include <linux/sdla_te1_pmc.h>
-#endif
-#endif
 
 /*
 *************************************************************************
@@ -184,21 +175,15 @@
 #define WAN_TE1_LIU_LLB_MODE	0x06
 #define WAN_TE1_LIU_RLB_MODE	0x07
 #define WAN_TE1_LIU_DLB_MODE	0x08
-#define WAN_TE1_FR_FLB_MODE	0x09
-#define WAN_TE1_FR_PLB_MODE	0x0A
-#define WAN_TE1_FR_RLB_MODE	0x0B
 #define WAN_TE1_LB_MODE_DECODE(mode)						\
-		((mode) == WAN_TE1_LINELB_MODE) ? "Line Loopback" :		\
+		((mode) == WAN_TE1_LINELB_MODE) ? "Line/Remote Loopback" :	\
 		((mode) == WAN_TE1_PAYLB_MODE) ? "Payload Loopback" :		\
 		((mode) == WAN_TE1_DDLB_MODE) ? "Diagnostic Digital Loopback" :	\
-		((mode) == WAN_TE1_TX_LB_MODE) ? "TX Loopback" :			\
+		((mode) == WAN_TE1_TX_LB_MODE) ? "TX Loopback" :		\
 		((mode) == WAN_TE1_LIU_ALB_MODE) ? "Analog LIU Loopback" :	\
-		((mode) == WAN_TE1_LIU_LLB_MODE) ? "Local LIU Loopback" :		\
+		((mode) == WAN_TE1_LIU_LLB_MODE) ? "Local LIU Loopback" :	\
 		((mode) == WAN_TE1_LIU_RLB_MODE) ? "Remote LIU Loopback" :	\
-		((mode) == WAN_TE1_LIU_DLB_MODE) ? "Dual LIU Loopback" :		\
-		((mode) == WAN_TE1_FR_FLB_MODE) ? "Framer Loopback" :		\
-		((mode) == WAN_TE1_FR_RLB_MODE) ? "Remote Loopback" :	\
-		((mode) == WAN_TE1_FR_PLB_MODE) ? "Payload Loopback" :	\
+		((mode) == WAN_TE1_LIU_DLB_MODE) ? "Dual LIU Loopback" :	\
 						"Unknown Loopback"
 
 /* Line loopback activate/deactive modes */
