@@ -91,6 +91,7 @@ static void display_Rx_code_condition(sdla_fe_t* fe);
 static int sdla_56k_print_alarm(sdla_fe_t* fe, unsigned int);
 static int sdla_56k_update_alarm_info(sdla_fe_t *fe, struct seq_file* m, int* stop_cnt);
 
+static int sdla_56k_polling(sdla_fe_t* fe);
 static int sdla_56k_unconfig(void* pfe);
 static int sdla_56k_intr(sdla_fe_t *fe);
 static int sdla_56k_check_intr(sdla_fe_t *fe);
@@ -267,6 +268,7 @@ int sdla_56k_iface_init(void *p_fe, void* pfe_iface)
 
 	fe_iface->config		= &sdla_56k_config;
 	fe_iface->unconfig		= &sdla_56k_unconfig;
+	fe_iface->polling		= &sdla_56k_polling;
 
 	fe_iface->get_fe_status		= &sdla_56k_get_fe_status;
 	fe_iface->get_fe_media		= &sdla_56k_get_fe_media;
@@ -298,6 +300,8 @@ static int sdla_56k_intr(sdla_fe_t *fe)
 	return 0;
 }
 
+
+
 /*	Called from ISR. On AFT card there is only on 56 port, it 
 	means the interrupt is always ours.
 	Returns: 1
@@ -413,6 +417,14 @@ static int sdla_56k_unconfig(void* pfe)
 	return 0;
 }
 
+static int sdla_56k_polling(sdla_fe_t* fe)
+{
+	
+	sdla_56k_alarm(fe, 1);
+
+	return 0;
+}
+
 static void display_Rx_code_condition(sdla_fe_t* fe)
 {
 	sdla_56k_param_t	*k56_param = &fe->fe_param.k56_param;
