@@ -12,6 +12,7 @@
  *		2 of the License, or (at your option) any later version.
  * ============================================================================
  * Oct 6, 2005	Alex Feldman	Initial version.
+ * Sep 06, 2008	Moises Silva    DAHDI support.
  ******************************************************************************
  */
 
@@ -27,9 +28,9 @@
 # include <wanpipe.h>
 # include <wanpipe_events.h>
 # include <sdla_remora.h>
-# include <zaptel.h>
+# include <zapcompat.h> /* Map of Zaptel -> DAHDI definitions */
 #else
-# include <zaptel.h>
+# include <zapcompat.h> /* Map of Zaptel -> DAHDI definitions */
 # include <linux/wanpipe_includes.h>
 # include <linux/wanpipe_defines.h>
 # include <linux/wanpipe.h>
@@ -130,6 +131,9 @@ typedef struct wp_tdmv_remora_ {
 
 	int		spanno;
 	struct zt_span	span;
+#ifdef DAHDI_ISSUES
+	struct zt_chan	*chans_ptrs[MAX_REMORA_MODULES];
+#endif
 	struct zt_chan	chans[MAX_REMORA_MODULES];
 	unsigned long	reg_module_map;	/* Registered modules */
 
@@ -1119,7 +1123,11 @@ static int wp_tdmv_remora_software_init(wan_tdmv_t *wan_tdmv)
 		}
 	}
 	wr->span.pvt = wr;
+#ifdef DAHDI_ISSUES
+	wr->span.chans		= wr->chans_ptrs;
+#else
 	wr->span.chans		= wr->chans;
+#endif
 	wr->span.channels	= num/*wr->max_timeslots*/;
 	wr->span.hooksig	= wp_remora_zap_hooksig;
 	wr->span.open		= wp_remora_zap_open;
@@ -1228,6 +1236,9 @@ static int wp_tdmv_remora_create(void* pcard, wan_tdmv_conf_t *tdmv_conf)
 	sdla_t			*card = (sdla_t*)pcard;
 	wp_tdmv_remora_t	*wr = NULL;
 	wan_tdmv_t		*tmp = NULL;
+#ifdef DAHDI_ISSUES
+	int i;
+#endif
 
 	WAN_ASSERT(card == NULL);
 	WAN_ASSERT(tdmv_conf->span_no == 0);
@@ -1272,6 +1283,11 @@ static int wp_tdmv_remora_create(void* pcard, wan_tdmv_conf_t *tdmv_conf)
 	wr->max_rxtx_len	= 0;
 	wan_spin_lock_irq_init(&wr->lockirq, "wan_rmtdmv_lock");
 	wan_spin_lock_irq_init(&wr->tx_rx_lockirq, "wan_rmtdmv_txrx_lock");
+#ifdef DAHDI_ISSUES
+	for (i = 0; i < sizeof(wr->chans)/sizeof(wr->chans[0]); i++) {
+		wr->chans_ptrs[i] = &wr->chans[i];
+	}
+#endif
 
 	if (tmp){
 		WAN_LIST_INSERT_AFTER(tmp, &card->wan_tdmv, next);
@@ -1868,14 +1884,26 @@ static void wp_tdmv_remora_dtmf (void* card_id, wan_event_t *event)
 
 	if (event->dtmf_type == WAN_EC_TONE_PRESENT){
 		wr->dtmfactive |= (1 << event->channel);
+#ifdef DAHDI_ISSUES
+		zt_qevent_lock(
+				wr->span.chans[event->channel-1],
+				(ZT_EVENT_DTMFDOWN | event->digit));
+#else
 		zt_qevent_lock(
 				&wr->span.chans[event->channel-1],
 				(ZT_EVENT_DTMFDOWN | event->digit));
+#endif
 	}else{
 		wr->dtmfactive &= ~(1 << event->channel);
+#ifdef DAHDI_ISSUES
+		zt_qevent_lock(
+				wr->span.chans[event->channel-1],
+				(ZT_EVENT_DTMFUP | event->digit));
+#else
 		zt_qevent_lock(
 				&wr->span.chans[event->channel-1],
 				(ZT_EVENT_DTMFUP | event->digit));
+#endif
 	}
 	return;
 }
