@@ -230,12 +230,15 @@ struct woomera_interface {
 	int q931_rel_cause_topbx;
 	int loop_tdm;
 	char session[SMG_SESSION_NAME_SZ];
+	int check_digits;	/* set to 1 when session comes up */
     	struct woomera_interface *next;
 };
 
 struct  woomera_session {
  	struct woomera_interface *dev;	
 	char session[SMG_SESSION_NAME_SZ];
+	char digits[MAX_DIALED_DIGITS+1];
+	int  digits_len;
 };
 
 #define CORE_TANK_LEN CORE_MAX_CHAN_PER_SPAN*CORE_MAX_SPANS
@@ -248,6 +251,7 @@ struct woomera_server {
 	pthread_mutex_t listen_lock;
 	pthread_mutex_t ht_lock;
 	pthread_mutex_t process_lock;
+	pthread_mutex_t digits_lock;
 	pthread_mutex_t media_udp_port_lock;
 	pthread_mutex_t thread_count_lock;
 	call_signal_connection_t mcon;
