@@ -28,6 +28,9 @@ sub new	{
 		_ss7_tdmchan => undef,
 		_ss7_subinterface => undef,
 		_ss7_tdminterface => undef,
+		_hp_option => undef,
+		_mtu_mru => undef,
+		_old_a10u => undef,
 	};			
 	bless $self, $class;
     	return $self;
@@ -122,6 +125,12 @@ sub ss7_option {
                  return $self->{_ss7_option};     
 }
 
+sub hp_option {
+          my ( $self, $hp_option ) = @_;
+                $self->{_hp_option} = $hp_option if defined($hp_option);
+                 return $self->{_hp_option};     
+}
+
 sub ss7_sigchan {
           my ( $self, $ss7_sigchan ) = @_;
                 $self->{_ss7_sigchan} = $ss7_sigchan if defined($ss7_sigchan);		
@@ -146,6 +155,18 @@ sub ss7_tdminterface {
 	        return $self->{_ss7_tdminterface};   
 }
 
+sub mtu_mru {
+          my ( $self, $mtu_mru ) = @_;
+                $self->{_mtu_mru} = $mtu_mru if defined($mtu_mru);		
+	        return $self->{_mtu_mru};     
+}
+
+sub old_a10u {
+          my ( $self, $old_a10u ) = @_;
+                $self->{_old_a10u} = $old_a10u if defined($old_a10u);		
+	        return $self->{_old_a10u};     
+}
+
 sub prompt_user{
 	my($promptString, $defaultValue) = @_;
 	if ($defaultValue) {
@@ -219,9 +240,22 @@ sub gen_wanpipe_ss7_subinterfaces{
 	my $device_no = $self->card->device_no;
 	my $ss7_subinterface = $self->ss7_subinterface;
 	my $ss7_tdmchan = $self->ss7_tdmchan;
+	my $mtu_mru = $self->mtu_mru;
 	my $hwec_mode = $self->card->hwec_mode;
 	my $ss7_tdminterface = $self->ss7_tdminterface;
+	my $hp_option	= $self->hp_option;
+	my $card_type="WAN_AFT_TE1";
+	my $old_a10u =  $self->old_a10u;
 	my $wanpipe_ss7_interfaces_template = $self->card->current_dir."/templates/ss7_a100/wanpipe.ss7.$ss7_subinterface";
+	if (!($hp_option == ' ')) {
+		#for hp tdmapi change templates 
+		$wanpipe_ss7_interfaces_template = $self->card->current_dir."/templates/hp_a100/wanpipe.hp.$ss7_subinterface";
+	}
+	
+	if(($old_a10u =~ m/YES/)){
+		$card_type="WAN_AFT";
+	}
+	
 	my $hw_dtmf = $self->card->hw_dtmf;
 	
 	open(FH, $wanpipe_ss7_interfaces_template) or die "Can't open $wanpipe_ss7_interfaces_template";
@@ -235,14 +269,15 @@ sub gen_wanpipe_ss7_subinterfaces{
 	close (FH);
 
 	open(FH, ">>$wanpipe_ss7_conf_file") or die "Cant open $wanpipe_ss7_conf_file";
+	$wp_file =~ s/WAN_AFT_TYPE/$card_type/g;
 	$wp_file =~ s/DEVNUM/$device_no/g;
 	$wp_file =~ s/SS7SIGCHAN/$ss7_sigchan/g;
 	$wp_file =~ s/TDMVOICECHAN/$ss7_tdmchan/g;
 	$wp_file =~ s/HWECMODE/$hwec_mode/g;
+	$wp_file =~ s/CHUNKTIME/$mtu_mru/g;
 	$wp_file =~ s/VOICEINTERFACE/$ss7_tdminterface/g;
 	$wp_file =~ s/TDMVSPANNO/$tdmv_span_no/g;
 	$wp_file =~ s/HWDTMF/$hw_dtmf/g;
-	
 	print FH $wp_file;
 	close (FH);
 }
@@ -265,8 +300,12 @@ sub gen_wanpipe_conf{
 	my $hwec_mode = $self->card->hwec_mode;
 	my $hw_dtmf = $self->card->hw_dtmf;
 	my $ss7_option = $self->ss7_option;
+	my $hp_option	= $self->hp_option;
 	my $dchan = $self->hw_dchan;
 	my $fe_lbo;
+	my $old_a10u =  $self->old_a10u;
+	my $fe_cpu = "A";
+	my $card_type="WAN_AFT_TE1";
 
 	my $device_alpha = &get_alpha_from_num($device_no);
 
@@ -276,8 +315,22 @@ sub gen_wanpipe_conf{
 	        $wanpipe_conf_template = $self->card->current_dir."/templates/ss7_a100/wanpipe.ss7.4";
 	} elsif ($ss7_option == 2){
                 $wanpipe_conf_template = $self->card->current_dir."/templates/ss7_a100/wanpipe.tdmvoiceapi.a100";
+	} elsif ($hp_option == 1){
+	        $wanpipe_conf_template = $self->card->current_dir."/templates/hp_a100/wanpipe.hp.4";
 	}
 	
+	if(($old_a10u =~ m/YES/)){
+		#Set card type and CPU for old card in for hpdmapi
+		$card_type="WAN_AFT";
+		if ($self->fe_line eq '1'){
+			$fe_cpu='A';
+		}elsif($self->fe_line eq '2'){
+			$fe_cpu='B';
+		}else{
+			print "Error: Invalid port on A101-2u\n";
+			exit 1;
+       		}
+	}
         $dchan = 0;
 	if(!$is_freebsd){
 		if ($self->signalling  =~ m/PRI/ | $self->signalling  =~ m/SS7/ ){
@@ -296,13 +349,13 @@ sub gen_wanpipe_conf{
 		$te_sig_mode_line= 'TE_SIG_MODE     = '.$te_sig_mode;
 	}
 
-	if($self->signalling eq 'TDM API'){
+	if($self->signalling eq 'TDM API' ){
                 $wanpipe_conf_template = $self->card->current_dir."/templates/wanpipe.tdm_api.a100";
 		#for tdm_api hw_dchan is set by user
 		$dchan = $self->hw_dchan;
 		
 	}
-
+	
 	open(FH, $wanpipe_conf_template) or die "Can't open $wanpipe_conf_template";
 	my $wp_file='';
        	while (<FH>) {
@@ -318,8 +371,10 @@ sub gen_wanpipe_conf{
 	} else {
 		$wp_file =~ s/IFNUM/$device_no/g;
 	}
-
-        $wp_file =~ s/SLOTNUM/$pci_slot/g;
+	
+	$wp_file =~ s/FECPU/$fe_cpu/g;
+	$wp_file =~ s/WAN_AFT_TYPE/$card_type/g;
+	$wp_file =~ s/SLOTNUM/$pci_slot/g;
         $wp_file =~ s/BUSNUM/$pci_bus/g;
         $wp_file =~ s/FEMEDIA/$fe_media/g;
         $wp_file =~ s/FELCODE/$fe_lcode/g;
@@ -333,7 +388,6 @@ sub gen_wanpipe_conf{
 	$wp_file =~ s/TDMVSPANNO/$tdmv_span_no/g;
         $wp_file =~ s/HWECMODE/$hwec_mode/g;
         $wp_file =~ s/HWDTMF/$hw_dtmf/g;
-	
 	print FH $wp_file;
 	close (FH);
 # print "\n created $fname for A$card_model $devnum SLOT $slot BUS $bus HWEC $hwec_mode\n";
