@@ -9,6 +9,7 @@ use strict;
 sub new	{
 	my ($class) = @_;
 	my $self = {
+		_is_tdm_api => undef,
 		_card      => undef,		
 		_tdm_opermode => 'FCC',
 		_tdm_law => 'MULAW',
@@ -43,6 +44,12 @@ sub rm_network_sync {
 				                        return $self->{_rm_network_sync};
 }
 
+sub is_tdm_api {
+	            my ( $self, $is_tdm_api ) = @_;
+		                    $self->{_is_tdm_api} = $is_tdm_api if defined($is_tdm_api);
+				                        return $self->{_is_tdm_api};
+}
+
 sub analog_modules {
 	            my ( $self, $analog_modules ) = @_;
 		                    $self->{_analog_modules} = $analog_modules if defined($analog_modules);
@@ -128,9 +135,15 @@ sub gen_wanpipe_conf{
 	my $rm_network_sync = $self->rm_network_sync;
 	my $hwec_mode = $self->card->hwec_mode;
 	my $hw_dtmf = $self->card->hw_dtmf;
+	my $is_tdm_api = $self->is_tdm_api;
+	my $tdm_voice_op_mode = "TDM_VOICE";
 
 	my $device_alpha = &get_alpha_from_num($device_no);
 
+	if($self->is_tdm_api eq '0') {
+		$tdm_voice_op_mode = "TDM_VOICE_API";
+	}
+	
 	open(FH, $wanpipe_conf_template) or die "Can open $wanpipe_conf_template";
 	my $wp_file='';
        	while (<FH>) {
@@ -146,7 +159,7 @@ sub gen_wanpipe_conf{
 		$wp_file =~ s/IFNUM/$device_no/g;
 	}
 
-
+	$wp_file =~ s/TDM_VOICE_OP_MODE/$tdm_voice_op_mode/g;
         $wp_file =~ s/SLOTNUM/$pci_slot/g;
         $wp_file =~ s/BUSNUM/$pci_bus/g;
         $wp_file =~ s/TDM_LAW/$tdm_law/g;
