@@ -1165,8 +1165,6 @@ int AFTUsage(void)
 	printf("\t             dlalb   Deactive LIU Analog Loopback mode (T1/E1 DM card only)\n");  
 	printf("\t             alllb   Active LIU Local Loopback mode (T1/E1 DM card only)\n");  
 	printf("\t             dlllb   Deactive LIU Local Loopback mode (T1/E1 DM card only)\n");  
-	printf("\t             alrlb   Active LIU Remote Loopback mode (T1/E1 DM card only)\n");  
-	printf("\t             dlrlb   Deactive LIU Remote Loopback mode (T1/E1 DM card only)\n");  
 	printf("\t             aldlb   Active LIU Dual Loopback mode (T1/E1 DM card only)\n");  
 	printf("\t             dldlb   Deactive LIU Dual Loopback mode (T1/E1 DM card only)\n");
 	printf("\t             allb3   Active Analog Local Loopback mode (DS3/E3 cards)\n");  
@@ -1370,27 +1368,18 @@ int AFTMain(char *command,int argc, char* argv[])
 				set_lb_modes(WAN_TE1_LIU_DLB_MODE, WAN_TE1_LB_ENABLE);
 			}else if (!strcmp(opt,"dldlb")){
 				set_lb_modes(WAN_TE1_LIU_DLB_MODE, WAN_TE1_LB_DISABLE);
-			}else if (!strcmp(opt,"aflb")){
-				set_lb_modes(WAN_TE1_FR_FLB_MODE, WAN_TE1_LB_ENABLE);
-			}else if (!strcmp(opt,"dflb")){
-				set_lb_modes(WAN_TE1_FR_FLB_MODE, WAN_TE1_LB_DISABLE);
-			}else if (!strcmp(opt,"afplb")){
-				set_lb_modes(WAN_TE1_FR_PLB_MODE, WAN_TE1_LB_ENABLE);
-			}else if (!strcmp(opt,"dfplb")){
-				set_lb_modes(WAN_TE1_FR_PLB_MODE, WAN_TE1_LB_DISABLE);
-
 			}else if (!strcmp(opt,"allb3")){
-				set_lb_modes(WAN_TE3_LIU_LB_ANALOG, WAN_TE1_LB_ENABLE);
+				set_lb_modes(WAN_TE3_LIU_LB_ANALOG, WAN_TE3_LB_ENABLE);
 			}else if (!strcmp(opt,"dllb3")){
-				set_lb_modes(WAN_TE3_LIU_LB_ANALOG, WAN_TE1_LB_DISABLE);
+				set_lb_modes(WAN_TE3_LIU_LB_ANALOG, WAN_TE3_LB_DISABLE);
 			}else if (!strcmp(opt,"arlb3")){
-				set_lb_modes(WAN_TE3_LIU_LB_REMOTE, WAN_TE1_LB_ENABLE);
+				set_lb_modes(WAN_TE3_LIU_LB_REMOTE, WAN_TE3_LB_ENABLE);
 			}else if (!strcmp(opt,"drlb3")){
-				set_lb_modes(WAN_TE3_LIU_LB_REMOTE, WAN_TE1_LB_DISABLE);
+				set_lb_modes(WAN_TE3_LIU_LB_REMOTE, WAN_TE3_LB_DISABLE);
 			}else if (!strcmp(opt,"adlb3")){
-				set_lb_modes(WAN_TE3_LIU_LB_DIGITAL, WAN_TE1_LB_ENABLE);
+				set_lb_modes(WAN_TE3_LIU_LB_DIGITAL, WAN_TE3_LB_ENABLE);
 			}else if (!strcmp(opt,"ddlb3")){
-				set_lb_modes(WAN_TE3_LIU_LB_DIGITAL, WAN_TE1_LB_DISABLE);
+				set_lb_modes(WAN_TE3_LIU_LB_DIGITAL, WAN_TE3_LB_DISABLE);
 			}else if (!strcmp(opt,"a")){
 				read_te1_56k_stat(0);
 			}else if (!strcmp(opt,"af")){
